ClassicRaidAssist = LibStub("AceAddon-3.0"):NewAddon("ClassicRaidAssist", "AceConsole-3.0", "AceHook-3.0", "AceEvent-3.0", "AceComm-3.0", "AceSerializer-3.0")
-- _G['ClassicRaidAssist'] = ClassicRaidAssist
_G['cooldownFrames'] = {}

local MAX_NUMBER_OF_UNITS = 100
local curr_unit_pointer = 1

local list_of_units = {}
for i=1, MAX_NUMBER_OF_UNITS do
    list_of_units[i] = nil
end


local announcePrefix = "CRA_announce"
local questionPrefix = "CRA_question"
local scanSendPrefix = "CRA_scansend"
local scanRecievePrefix = "CRA_scanrecieve"
local questionItemPrefix = "CRA_questionitem"
local sendItemPrefix = "CRA_senditem"
local sendBadItemPrefix = "CRA_sendbaditem"

local version = GetAddOnMetadata("ClassicRaidAssist", "Version")

local name, _ = UnitName("player")

local hasScanned = false

function ClassicRaidAssist:Scan()
    hasScanned = true
    self.raid[name] = version
    ClassicRaidAssist:SendMessage(version, scanSendPrefix)
end

function ClassicRaidAssist:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("ClassicRaidAssistDB", {
        factionrealm = {
            minimapButton = {hide = false}
        },
        char = {
            buffsToCheckFor = {},
            worldBuffsToCheckFor = {},
            CurrentRaidBuffsOutputChannel = "RAID",
            CurrentWorldBuffsOutputChannel = "RAID",
            CurrentRaidItemsOutputChannel = "RAID",
            CurrentRaidItemCheck = "Aqual Quintessence",
            allowItemScanning = true,
            raidItemsContainer = nil,
            settingsContainer = nil,
            currentTab = "raidBuffs"
        }
    }, true)
    
    self.raid = {}
    self.unit = {}
    
    self:RegisterComm(announcePrefix, "OnAnnounce")
    self:RegisterComm(questionPrefix, "OnQuestion")
    self:RegisterComm(scanSendPrefix, "OnScanSend")
    self:RegisterComm(scanRecievePrefix, "OnScanRecieve")
    self:RegisterComm(questionItemPrefix, "OnQuestionItem")
    self:RegisterComm(sendItemPrefix, "OnSendItem")
    self:RegisterComm(sendBadItemPrefix, "OnSendBadItem")
    ClassicRaidAssist:SetUpMouseEvent()
    ClassicRaidAssistGUI:SetUpGUI()
    ClassicRaidAssist:SetUpMinimapIcon()
    
    local options = {
        name = 'ClassicRaidAssist',
        type = 'group'
    }
    LibStub("AceConfig-3.0"):RegisterOptionsTable("ClassicRaidAssist", options)
end 

function ClassicRaidAssist:ClearCooldowns()
    for key, value in pairs(cooldownFrames) do
        value:Hide()
        tremove(cooldownFrames, key)
    end
end

function ClassicRaidAssist:SendMessage(msg, prefix)
    if(IsInRaid()) then
        ClassicRaidAssist:SendCommMessage(prefix, msg, "RAID")
    elseif(IsInGroup()) then
        ClassicRaidAssist:SendCommMessage(prefix, msg, "PARTY")
    end
end

function ClassicRaidAssist:DetermineLooter(guid)
    canLoot, inRange = CanLootUnit(guid)
    if canLoot then
        local localizedClass, englishClass, classIndex = UnitClass("player")
        ClassicRaidAssist:SendMessage(guid, announcePrefix)
        return name, englishClass, false
    else
        ClassicRaidAssist:SendMessage(guid, questionPrefix)
        return _, _, true
    end
end

function ClassicRaidAssist:ScanForItem(message)
    ClassicRaidAssist:SendMessage(message, questionItemPrefix)
end

function ClassicRaidAssist:OnQuestionItem(prefix, message, distribution, sender)
    if not ClassicRaidAssist.db.char.allowItemScanning then
        return
    end

    for key, value in pairs(RaidItemsTab.raidItems) do
        if (key == message) then 
            for k, v in pairs(value['itemIds']) do
                count  = GetItemCount(v)
                if count > 0 then
                    ClassicRaidAssist:SendMessage(message, sendItemPrefix)
                else
                    ClassicRaidAssist:SendMessage(message, sendBadItemPrefix)
                end
            end
        end
    end
end

function ClassicRaidAssist:OnSendItem(prefix, message, distribution, sender)
    for key, value in pairs(RaidItemsTab.raidItems) do
        if(key == message) then
            value['hasThisItem'][sender] = true
            if(ClassicRaidAssist.db.char.currentTab == "raidItems") then
                ClassicRaidAssist.db.char.raidItemsContainer:ReleaseChildren()
                RaidItemsTab:DrawRaidItems(ClassicRaidAssist.db.char.raidItemsContainer)
            end
            break
        end
    end
end

function ClassicRaidAssist:OnSendBadItem(prefix, message, distribution, sender)
    for key, value in pairs(RaidItemsTab.raidItems) do
        if(key == message) then
            value['hasThisItem'][sender] = false
            if(ClassicRaidAssist.db.char.currentTab == "raidItems") then
                ClassicRaidAssist.db.char.raidItemsContainer:ReleaseChildren()
                RaidItemsTab:DrawRaidItems(ClassicRaidAssist.db.char.raidItemsContainer)
            end
            break
        end
    end
end

function ClassicRaidAssist:OnQuestion(prefix, message, distribution, sender)
    canLoot, inRange = CanLootUnit(message)
    if canLoot then
        local localizedClass, englishClass, classIndex = UnitClass("player")
        ClassicRaidAssist:SendMessage(message, announcePrefix)
        ClassicRaidAssist:AddToUnits(message, name, englishClass)
    end
end

function ClassicRaidAssist:OnAnnounce(prefix, message, distribution, sender)
    local localizedClass, englishClass, classIndex = UnitClass(sender)
    ClassicRaidAssist:AddToUnits(message, sender, englishClass)
end

function ClassicRaidAssist:OnScanSend(prefix, message, distribution, sender)
    -- if(sender ~= name) then
    ClassicRaidAssist:SendMessage(version, scanRecievePrefix)
    -- end
end

function ClassicRaidAssist:OnScanRecieve(prefix, message, distribution, sender)
    self.raid[sender] = message
    if(ClassicRaidAssist.db.char.currentTab == "settings") then
        ClassicRaidAssist.db.char.settingsContainer:ReleaseChildren()
        ClassicRaidAssistGUI:DrawSettings(ClassicRaidAssist.db.char.settingsContainer)
    end
end

function ClassicRaidAssist:AddToUnits(guid, looter, class)
    if(list_of_units[curr_unit_pointer] ~= nil)  then
        ClassicRaidAssist.unit[list_of_units[curr_unit_pointer]] = nil
    end
    
    ClassicRaidAssist.unit[guid] = {
        ["looter"] = looter,
        ["unitClassColor"] = RAID_CLASS_COLORS[class].colorStr
    }
    list_of_units[curr_unit_pointer] = guid
    curr_unit_pointer = curr_unit_pointer + 1
    if(curr_unit_pointer > MAX_NUMBER_OF_UNITS) then
        curr_unit_pointer = 1
    end
end

function ClassicRaidAssist:SetUpMouseEvent()
    local frame = CreateFrame("Frame")
    frame:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
    frame:SetScript("OnEvent", function(self)
        if(IsInRaid() or IsInGroup()) then
            local name = UnitName("mouseover")
            local guid = UnitGUID("mouseover")
            local friend = UnitIsFriend("mouseover", "player")
            local dead = UnitIsDead("mouseover")
            local creatureType = UnitCreatureType("mouseover")
            local tapped = UnitIsTapDenied("mouseover")
            
            if name == nil or friend or not dead or creatureType == "Critter" or tapped then return end
            
            if ClassicRaidAssist.unit[guid] == nil then
                playerName, playerClass, not_found = ClassicRaidAssist:DetermineLooter(guid)
            
                if not not_found then
                    ClassicRaidAssist:AddToUnits(guid, playerName, playerClass)
                end
            end
            
            if ClassicRaidAssist.unit[guid] == nil then
                GameTooltip:AddLine("Looter: |cababababUnknown|r", 1, 1, 1)
                GameTooltip:Show()
            else
                GameTooltip:AddLine("Looter: |c" .. ClassicRaidAssist.unit[guid].unitClassColor .. ClassicRaidAssist.unit[guid].looter .. "|r", 1, 1, 1)
                GameTooltip:Show()
            end
        end
    end)
end

function ClassicRaidAssist:SetUpMinimapIcon()
	LibStub("LibDBIcon-1.0"):Register("ClassicRaidAssist", LibStub("LibDataBroker-1.1"):NewDataObject("ClassicRaidAssist",
	{
		type = "data source",
		text = "Classic Raid Assist",
		icon = "Interface\\Icons\\ability_creature_disease_02",
		OnClick = function(self, button) 
			if (button == "LeftButton") then
				ClassicRaidAssistGUI:Toggle()
            end
		end,
		OnTooltipShow = function(tooltip)
			tooltip:AddDoubleLine(format("%s", "Classic Raid Assist"), format("|cff777777v%s", GetAddOnMetadata("ClassicRaidAssist", "Version")));
			tooltip:AddLine("|cFFCFCFCFLeft Click: |rShow GUI");
		end
	}), self.db.factionrealm.minimapButton);
end